package org.basex.query.expr;

import org.basex.query.QueryContext;
import org.basex.query.QueryException;
import org.basex.query.item.Item;
import org.basex.query.item.QNm;
import org.basex.query.item.Str;
import org.basex.query.iter.Iter;
import org.basex.query.util.Var;
import org.basex.util.Token;

/**
 * Catch clause.
 *
 * @author Workgroup DBIS, University of Konstanz 2005-10, ISC License
 * @author Christian Gruen
 */
public final class Catch extends Single {
  /** Variable. */
  private final Var[] var;
  /** Supported codes. */
  private final QNm[] codes;

  /**
   * Constructor.
   * @param ct catch expression
   * @param c supported error codes
   * @param v variables
   */
  public Catch(final Expr ct, final QNm[] c, final Var... v) {
    super(ct);
    codes = c;
    var = v;
  }

  @Override
  public Catch comp(final QueryContext ctx) throws QueryException {
    final int s = ctx.vars.size();
    for(final Var v : var) ctx.vars.add(v);
    super.comp(ctx);
    ctx.vars.reset(s);
    return this;
  }

  /**
   * Catch iterator.
   * @param ctx query context
   * @param ex thrown exception
   * @return resulting item
   * @throws QueryException query exception
   */
  Iter iter(final QueryContext ctx, final QueryException ex)
      throws QueryException {

    final String cd = ex.code();
    final byte[] code = cd == null ? Token.EMPTY : Token.token(cd);
    if(!find(code)) return null;

    final int s = ctx.vars.size();
    if(var.length > 0) {
      ctx.vars.add(var[0].bind(new QNm(code), ctx).copy());
    }
    if(var.length > 1) {
      ctx.vars.add(var[1].bind(Str.get(ex.simple()), ctx).copy());
    }
    if(var.length > 2) {
      ctx.vars.add(var[2].bind(ex.iter.finish(), ctx).copy());
    }
    final Item it = ctx.iter(expr).finish();
    ctx.vars.reset(s);
    return it.iter();
  }

  /**
   * Finds iterator.
   * @param err error code
   * @return result of check
   */
  private boolean find(final byte[] err) {
    for(final QNm c : codes) if(c == null || Token.eq(c.ln(), err)) return true;
    return false;
  }

  @Override
  public boolean uses(final Use u, final QueryContext ctx) {
    return u == Use.VAR || super.uses(u, ctx);
  }

  @Override
  public Expr remove(final Var v) {
    for(final Var vr : var) if(!v.visible(vr)) return this;
    return super.remove(v);
  }

  @Override
  public String toString() {
    return "catch { " + expr + "}";
  }
}
